package gus05.entity.gus.security.asymkeys.checkkeyfiles;



import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.Properties;

import gus05.framework.core.Entity;
import gus05.framework.core.Outside;
import gus05.framework.core.Service;
import gus05.framework.features.Give;



public class CheckKeyFiles implements Entity, Give {

    
    public String getName() {return "gus.security.asymkeys.checkkeyfiles";}
    public String getCreationDate() {return "2008.06.10";}
    
    
    
    private Service fileToPublicKey;
    private Service fileToPrivateKey;
    
    
    public CheckKeyFiles() throws Exception
    {
    	fileToPublicKey = Outside.service(this,"gus.security.asymkeys.filetopublickey");
    	fileToPrivateKey = Outside.service(this,"gus.security.asymkeys.filetoprivatekey");
    }

    
    
    public void give(Object obj) throws Exception
	{
    	File[] files = (File[]) obj;
    	if(files.length!=2) e("wrong file number: "+files.length);
    	
    	File f1 = files[0];
    	File f2 = files[1];
    	
    	if(!f1.exists()) e("file not found: "+f1.getAbsolutePath());
    	if(!f2.exists()) e("file not found: "+f2.getAbsolutePath());
    	
    	String name1 = getName0(f1);
    	String name2 = getName0(f2);
    	
    	if(!name1.equals(name2))
    		e("wrong names for files "+f1.getName()+" and "+f2.getName());
    	
    	f1.setReadOnly();
    	f2.setReadOnly();
    	
    	
    	
    	
    	
    	File file_pr = null;
    	File file_pu = null;
    	
    	Properties prop_pr = null;
    	Properties prop_pu = null;
    	
    	
    	if(isPrivateKey(f1) && isPublicKey(f2))
    	{
    		file_pr = f1;
    		prop_pr = prop(f1);
    		file_pu = f2;
    		prop_pu = prop(f2);
    	}
    	else if(isPrivateKey(f2) && isPublicKey(f1))
    	{
    		file_pr = f2;
    		prop_pr = prop(f2);
    		file_pu = f1;
    		prop_pu = prop(f1);
    	}
    	else e("wrong extensions for files "+f1.getName()+" and "+f2.getName());
    	
    	
    	checkSame(prop_pr,prop_pu,SECURITY.PUBLICKEY_Y0);
    	checkSame(prop_pr,prop_pu,SECURITY.PUBLICKEY_P0);
    	checkSame(prop_pr,prop_pu,SECURITY.PUBLICKEY_Q0);
    	checkSame(prop_pr,prop_pu,SECURITY.PUBLICKEY_G0);
    	checkSame(prop_pr,prop_pu,SECURITY.PUBLICKEY_SIGN);
    	checkSame(prop_pr,prop_pu,SECURITY.PUBLICKEY_OWNER,name1);
    	
    	
    	
    	
    	fileToPrivateKey.transform(file_pr);
    	fileToPublicKey.transform(file_pu);
	}
    
    
    
    
    
    private boolean isPrivateKey(File file)
    {return file.getName().endsWith("."+SECURITY.PRIVATEKEY_FILEEXTENSION);}
    
    private boolean isPublicKey(File file)
    {return file.getName().endsWith("."+SECURITY.PUBLICKEY_FILEEXTENSION);}
    
    
    
    private String getName0(File file)
    {
        String name = file.getName();
        if(!name.contains(".")) return name;
        String[] n = name.split("\\.");
        int l = n[n.length-1].length();
        return name.substring(0,name.length()-1-l);
    }
    
    
    
    
    private void checkSame(Properties p1, Properties p2, String key) throws Exception
    {
    	if(!p1.containsKey(key)) throw new Exception("property "+key+" not found");
    	if(!p2.containsKey(key)) throw new Exception("property "+key+" not found");
    	String v1 = p1.getProperty(key);
    	String v2 = p2.getProperty(key);
    	if(!v1.equals(v2)) throw new Exception("different values for property "+key);
    }
    
    
    private void checkSame(Properties p1, Properties p2, String key, String value) throws Exception
    {
    	if(!p1.containsKey(key)) throw new Exception("property "+key+" not found");
    	if(!p2.containsKey(key)) throw new Exception("property "+key+" not found");
    	String v1 = p1.getProperty(key);
    	String v2 = p2.getProperty(key);
    	if(!v1.equals(value)) throw new Exception("incorrect value for property "+key);
    	if(!v1.equals(v2)) throw new Exception("different values for property "+key);
    }
    
    
    
    
    
    
    private Properties prop(File file) throws IOException
    {
        Properties prop = new Properties();
        FileInputStream fis = new FileInputStream(file);
        prop.load(fis);
        fis.close();
        return prop;
    }
    
    
    
    
    private void e(String message) throws Exception
    {throw new Exception(message);}
} 
