package gus05.entity.gus.security.asymkeys.publickey.signverifier;


import java.security.PublicKey;
import java.security.Signature;

import gus05.framework.core.Entity;
import gus05.framework.core.Outside;
import gus05.framework.core.Service;
import gus05.framework.features.Filter;
import gus05.framework.features.Transform;



public class PublicKeySignVerifier implements Entity, Transform {


    public String getName()				{return "gus.security.asymkeys.publickey.signverifier";}
    public String getCreationDate()		{return "2008.07.01";}


    
    private Service base64Decoder;
    private Signature si;
    


    public PublicKeySignVerifier() throws Exception
    {
    	base64Decoder = Outside.service(this,"gus.tobytearray.base64tobyte");
    	si = Signature.getInstance(SECURITY.SECURITY_ALGO_DSA);
    }



    public Object transform(Object obj) throws Exception
    {
    	PublicKey key = (PublicKey) obj;
        return new SignVerifier(key);
    }


    
    
    private class SignVerifier implements Filter
    {
    	private PublicKey key;
    	public SignVerifier(PublicKey key) {this.key = key;}
    	
		public boolean filter(Object obj) throws Exception
		{
			Object[] info = (Object[]) obj;
			if(info.length!=2) throw new Exception("wrong info number: "+info.length);
			
			String message = (String) info[0];
			String sign = (String) info[1];
			
			return verifySign(key,message,sign);
		}
    	
    }
    
    
    
    
    
    private boolean verifySign(PublicKey publicKey, String message, String sign) throws Exception
    {
    	si.initVerify(publicKey);
        si.update(message.getBytes());
        byte[] signData = decode(sign);
        return si.verify(signData);
    }
    
    

    private byte[] decode(String s) throws Exception
    {return (byte[]) base64Decoder.transform(s);}
}

