package gus05.entity.gus.security.asymkeys.signedmessage.editor1;


import gus05.framework.core.Entity;
import gus05.framework.core.Outside;
import gus05.framework.core.Service;
import gus05.framework.features.Give;
import gus05.framework.features.Provide;
import gus05.framework.features.Graphic;
import gus05.framework.features.Transform;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JComponent;
import javax.swing.JTextPane;
import javax.swing.KeyStroke;
import javax.swing.SwingUtilities;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.UndoableEditEvent;
import javax.swing.text.BadLocationException;
import javax.swing.text.SimpleAttributeSet;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyledDocument;
import javax.swing.undo.UndoManager;

import java.awt.Color;
import java.awt.Event;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.security.PrivateKey;



public class SignedMessageEditor1 implements Entity, Give, Provide, Graphic, DocumentListener {


    public String getName()				{return "gus.security.asymkeys.signedmessage.editor1";}
    public String getCreationDate()		{return "2008.07.01";}

    
    public static final String BLANK_MESSAGE = "\n\n\n";
    
    private UndoManager undo;
    private KeyStroke undoKey;
    private Action undoAction;

    private Service signerBuilder;
    private Transform signer;
    
    private JTextPane0 area;

    private PrivateKey privateKey;
    
    private String owner;
    private String message;
    private String sign;

    

    public SignedMessageEditor1() throws Exception
    {
    	signerBuilder = Outside.service(this,"gus.security.asymkeys.privatekey.signer");
    	
    	undo = new UndoManager();
    	undoKey = KeyStroke.getKeyStroke(KeyEvent.VK_Z,Event.CTRL_MASK);
    	undoAction = new AbstractAction(){
			public void actionPerformed(ActionEvent e)
			{undo();}
		};
		
		
    	area = new JTextPane0();
    	area.getDocument().addDocumentListener(this);
    	area.getDocument().addUndoableEditListener(undo);
    	area.getInputMap().put(undoKey, undoAction);
    }



    public void give(Object obj) throws Exception
    {
    	Object[] info = (Object[]) obj;
    	if(info.length!=2) throw new Exception("wrong info number: "+info.length);
    	
    	privateKey = (PrivateKey) info[0];
    	owner = (String) info[1];
    	
    	signer = (Transform) signerBuilder.transform(privateKey);
    	area.setText(BLANK_MESSAGE);
    }



    public JComponent gui()
    {return area;}


    public Object provide() throws Exception
    {return privateKey;}
    
    
    
    
    private void undo()
    {
    	if(undo.canUndo())
    	undo.undo();
    }
    
    
    
    
    
    private void updateArea()
    {
    	try
    	{
    		area.getDocument().removeDocumentListener(this);
    		area.getDocument().removeUndoableEditListener(undo);
    		
    		int p = area.getCaretPosition();
    		area.setText("");
    		area.appendText(message);
    		area.appendBoldText("\n"+owner+"\n"+sign);
    		int length = area.getText().length();
    		if(p>length) p=length;
            area.setCaretPosition(p);
    		
            area.getDocument().addDocumentListener(this);
    		area.getDocument().addUndoableEditListener(undo);
    	}
    	catch(Exception e)
    	{Outside.err(this,"updateArea()",e);}
    }
    
    
    
    
    
    
    public void changedUpdate(DocumentEvent e) {}
    public void insertUpdate(DocumentEvent e) {areaEdited();}
    public void removeUpdate(DocumentEvent e) {areaEdited();}
    
    public void undoableEditHappened(UndoableEditEvent evt)
    {undo.addEdit(evt.getEdit());}

    
    
    
    

    private void areaEdited()
    {
        SwingUtilities.invokeLater(new Runnable(){
            public void run()
            {performChange();}
        });
    }
    
    
    private void performChange()
    {
        try
        {
        	if(privateKey==null) return;
        	
        	// find message part
            String text = area.getText().replace("\r\n","\n");
            String _sign = "\n"+sign;
            String _owner = "\n"+owner;
            
            String t = trim(text);
            if(sign!=null && t.endsWith(_sign))
            	text = t.substring(0,t.length()-_sign.length());
            
            t = trim(text);
            if(owner!=null && t.endsWith(_owner))
            	text = t.substring(0,t.length()-_owner.length());
            
            message = text;
            
            // generate sign
            sign = (String) signer.transform(message+"\n"+owner);
    		
    		updateArea();
        }
        catch(Exception e)
        {Outside.err(this,"performChange()",e);}
    }
    
    
    
    
    private String trim(String s)
    {
    	StringBuffer b = new StringBuffer(s);
    	while(b.length()>0 && b.charAt(b.length()-1)=='\n')
    		b.deleteCharAt(b.length()-1);
    	return b.toString();
    }
    
    
    
    
    
    private class JTextPane0 extends JTextPane
    {
        private StyledDocument doc;
        private SimpleAttributeSet attr;
        
        public void initBold(boolean val)
        {StyleConstants.setBold(attr, val);}
        
        public void initItalic(boolean val)
        {StyleConstants.setItalic(attr, val);}
         
        public void initUnderline(boolean val)
        {StyleConstants.setUnderline(attr, val);}
        
        public void initForeground(Color color)
        {StyleConstants.setForeground(attr, color);}
        
        public void initBackground(Color color)
        {StyleConstants.setBackground(attr, color);}
        
        public void appendText(String text) throws BadLocationException
        {doc.insertString(doc.getLength(),text,attr);}
        
        public void appendBoldText(String text) throws BadLocationException
        {
            initBold(true);
            doc.insertString(doc.getLength(),text,attr);
            initBold(false);
        }
        
        public JTextPane0()
        {
            super();
            doc = getStyledDocument();
            attr = new SimpleAttributeSet();
        }
        
        public void positionToEnd()
        {setCaretPosition(doc.getLength());}
    }
}

